/*
 * Copyright (c) Citrix Systems, Inc.
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of version 2 of the GNU General Public License as published
 * by the Free Software Foundation, with the additional linking exception as
 * follows:
 *
 *   Linking this library statically or dynamically with other modules is
 *   making a combined work based on this library. Thus, the terms and
 *   conditions of the GNU General Public License cover the whole combination.
 *
 *   As a special exception, the copyright holders of this library give you
 *   permission to link this library with independent modules to produce an
 *   executable, regardless of the license terms of these independent modules,
 *   and to copy and distribute the resulting executable under terms of your
 *   choice, provided that you also meet, for each linked independent module,
 *   the terms and conditions of the license of that module. An independent
 *   module is a module which is not derived from or based on this library. If
 *   you modify this library, you may extend this exception to your version of
 *   the library, but you are not obligated to do so. If you do not wish to do
 *   so, delete this exception statement from your version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

using System;
using System.Collections.Generic;
using System.Text;
using System.Resources;
using System.Collections;
using System.Text.RegularExpressions;
using System.Xml;

namespace XenAPI
{
    public partial class Failure : Exception
    {
        public const string INTERNAL_ERROR = "INTERNAL_ERROR";
        public const string MESSAGE_PARAMETER_COUNT_MISMATCH = "MESSAGE_PARAMETER_COUNT_MISMATCH";

        private static ResourceManager errorDescriptions = XenAPI.FriendlyErrorNames.ResourceManager;

        private readonly List<string> errorDescription;
        private String errorText;
        private String shortError;

        public List<string> ErrorDescription 
        {
            get 
            {
                return errorDescription;
            }
        }

        public string ShortMessage
        {
            get
            {
                return shortError;
            }
        }

        public override string Message
        {
            get
            {
                return errorText;
            }
        }

        public Failure(params string[] err) : this(new List<string>(err))
        {
        }

        public Failure(List<string> ErrorDescription)
        {
            this.errorDescription = ErrorDescription;
            Setup();
        }

        public void Setup()
        {
            if (ErrorDescription.Count > 0)
            {
                try
                {
                    string formatString;
                    try
                    {
                        formatString = errorDescriptions.GetString(ErrorDescription[0]);
                    }
                    catch
                    {
                        formatString = null;
                    }

                    if (formatString == null)
                    {
                        // If we don't have a translation, just combine all the error results from the server
                        List<string> cleanBits = new List<string>();
                        foreach (string s in ErrorDescription)
                        {
                            // Only show non-empty bits of ErrorDescription.
                            // Also, trim the bits, since the server occasionally sends spurious newlines.
                            if (s.Trim().Length > 0)
                            {
                                cleanBits.Add(s.Trim());
                            }
                        }

                        this.errorText = string.Join(" - ", cleanBits.ToArray());
                    }
                    else
                    {

                        // We need a string array to pass to String.Format, and it must not contain the 0th element.

                        string[] objects = new string[ErrorDescription.Count - 1];

                        for (int i = 1; i < ErrorDescription.Count; i++)
                            objects[i - 1] = ErrorDescription[i];

                        this.errorText = String.Format(formatString, objects);
                    }
                }
                catch (Exception)
                {
                    this.errorText = ErrorDescription[0];
                }

                try
                {
                    shortError = errorDescriptions.GetString(ErrorDescription[0] + "-SHORT");
                }
                catch (Exception)
                {
                    shortError = this.errorText;
                }

                // now try and parse CSLG failures (these have embedded xml)
                TryParseCslg();
            }
        }

        /// <summary>
        /// Tries the parse CSLG failures. These have embedded xml. The useful part (from the user's perspective) is copied to errorText.
        /// </summary>
        /// <returns>A value specifying whether a CSLG error was found.</returns>
        private bool TryParseCslg()
        {
            //failure.ErrorDescription[2]:
            //<StorageLinkServiceError>
            //    <Fault>Host ivory has not yet been added to the service. [err=Object was not found]</Fault>
            //    <Detail>
            //        <errorCode>6</errorCode>
            //        <messageId></messageId>
            //        <defaultMessage>Host ivory has not yet been added to the service. [err=Object was not found]</defaultMessage>
            //        <severity>2</severity>
            //        <errorFunction>CXSSHostUtil::getHost</errorFunction>
            //        <errorLine>113</errorLine>
            //        <errorFile>.\\xss_util_host.cpp</errorFile>
            //    </Detail>
            // </StorageLinkServiceError>

            if (ErrorDescription.Count > 2 && ErrorDescription[2] != null && ErrorDescription[0] != null && ErrorDescription[0].StartsWith("SR_BACKEND_FAILURE"))
            {
                Match m = Regex.Match(ErrorDescription[2], @"<StorageLinkServiceError>.*</StorageLinkServiceError>", RegexOptions.Singleline);

                if (m.Success)
                {
                    XmlDocument doc = new XmlDocument();

                    try
                    {
                        doc.LoadXml(m.Value);
                    }
                    catch (XmlException)
                    {
                        return false;
                    }

                    XmlNodeList nodes = doc.SelectNodes("/StorageLinkServiceError/Fault");

                    if (nodes != null && nodes.Count > 0 && !string.IsNullOrEmpty(nodes[0].InnerText))
                    {
                        if (string.IsNullOrEmpty(errorText))
                        {
                            errorText = nodes[0].InnerText;
                        }
                        else
                        {
                            errorText = string.Format("{0} ({1})", errorText, nodes[0].InnerText);
                        }
                        return true;
                    }
                }
            }
            return false;
        }

        public override string ToString()
        {
            return Message;
        }
    }
}
