/*
 * Copyright (c) Citrix Systems, Inc.
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of version 2 of the GNU General Public License as published
 * by the Free Software Foundation, with the additional linking exception as
 * follows:
 *
 *   Linking this library statically or dynamically with other modules is
 *   making a combined work based on this library. Thus, the terms and
 *   conditions of the GNU General Public License cover the whole combination.
 *
 *   As a special exception, the copyright holders of this library give you
 *   permission to link this library with independent modules to produce an
 *   executable, regardless of the license terms of these independent modules,
 *   and to copy and distribute the resulting executable under terms of your
 *   choice, provided that you also meet, for each linked independent module,
 *   the terms and conditions of the license of that module. An independent
 *   module is a module which is not derived from or based on this library. If
 *   you modify this library, you may extend this exception to your version of
 *   the library, but you are not obligated to do so. If you do not wish to do
 *   so, delete this exception statement from your version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

using System;
using System.Collections.Generic;
using System.Text;
using System.Collections;
using System.Reflection;

namespace XenAPI
{
    public class Marshalling
    {
        /// <summary>
        /// Takes a Hashtable, creates a new t, and populates the fields of 
        /// that t with the values from the Hashtable.
        /// </summary>
        /// <param name="t">A XenAPI type, VM for example.  t must have an associated XenAPI.Proxy_t.</param>
        /// <param name="table"></param>
        /// <returns></returns>
        public static object convertStruct(Type t, Hashtable table)
        {
            return t.GetConstructor(new Type[] { typeof(Hashtable) }).Invoke(new object[] { table });
        }

        public static Type GetXenAPIType(string name)
        {
            return Type.GetType(string.Format("XenAPI.{0}", name), false, true);
        }

        public static bool ParseBool(Hashtable table, string key)
        {
            return table.ContainsKey(key) ? (bool)table[key] : false;
        }

        public static DateTime ParseDateTime(Hashtable table, string key)
        {
            return table.ContainsKey(key) ? (DateTime)table[key] : DateTime.MinValue;
        }

        public static double ParseDouble(Hashtable table, string key)
        {
            return table.ContainsKey(key) ? (double)table[key] : 0.0;
        }

        public static Hashtable ParseHashTable(Hashtable table, string key)
        {
            return table.ContainsKey(key) ? (Hashtable)table[key] : null;
        }

        public static long ParseLong(Hashtable table, string key)
        {
            string val = table.ContainsKey(key) ? (string)table[key] : null;
            return val == null ? 0 : long.Parse(val);
        }

        public static string ParseString(Hashtable table, string key)
        {
            return table.ContainsKey(key) ? (string)table[key] : null;
        }

        public static string[] ParseStringArray(Hashtable table, string key)
        {
            object[] val = table.ContainsKey(key) ? (object[])table[key] : null;
            if (val == null)
            {
                return new string[0];
            }
            else
            {
                return Array.ConvertAll<object, string>(val, delegate(object o) { return o.ToString(); });
            }
        }

        public static XenRef<T> ParseRef<T>(Hashtable table, string key) where T : XenObject<T>
        {
            string r = ParseString(table, key);
            return r == null ? null : XenRef<T>.Create(r);
        }

        public static List<XenRef<T>> ParseSetRef<T>(Hashtable table, string key) where T : XenObject<T>
        {
            string[] rs = ParseStringArray(table, key);
            return rs == null ? null : XenRef<T>.Create(rs);
        }

        public static Dictionary<XenRef<T>, T> ParseMapRefRecord<T, U>(Hashtable table, string key) where T : XenObject<T>
        {
            Hashtable map = ParseHashTable(table, key);
            return map == null ? null : XenRef<T>.Create<U>(map);
        }
    }
}
