/*
 * Copyright (c) Citrix Systems, Inc.
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of version 2 of the GNU General Public License as published
 * by the Free Software Foundation, with the additional linking exception as
 * follows:
 *
 *   Linking this library statically or dynamically with other modules is
 *   making a combined work based on this library. Thus, the terms and
 *   conditions of the GNU General Public License cover the whole combination.
 *
 *   As a special exception, the copyright holders of this library give you
 *   permission to link this library with independent modules to produce an
 *   executable, regardless of the license terms of these independent modules,
 *   and to copy and distribute the resulting executable under terms of your
 *   choice, provided that you also meet, for each linked independent module,
 *   the terms and conditions of the license of that module. An independent
 *   module is a module which is not derived from or based on this library. If
 *   you modify this library, you may extend this exception to your version of
 *   the library, but you are not obligated to do so. If you do not wish to do
 *   so, delete this exception statement from your version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

using System;
using System.Collections.Generic;
using System.Text;

namespace XenAPI
{
    public class UserDetails
    {
        // Very large group memberships cause us to hang on connection time as the get subject info call can take some time.
        private static readonly int MAX_GROUP_LOOKUP = 40;

        /// <summary>
        /// Mapping of SIDS to UserDetails.
        /// </summary>
        private static Dictionary<string, UserDetails> sid_To_UserDetails = new Dictionary<string,UserDetails>();
        public static void UpdateDetails(string SID, Session session)
        {
            lock (UserDetails.sid_To_UserDetails)
            {
                UserDetails.sid_To_UserDetails.Remove(SID);
                UserDetails.sid_To_UserDetails.Add(SID, new UserDetails(session));
            }
        }
        public static Dictionary<string, UserDetails> Sid_To_UserDetails
        {
            get
            {
                lock (UserDetails.sid_To_UserDetails)
                {
                    return sid_To_UserDetails;
                }
            }
        }

        private string userSid = null;
        private string userDisplayName = null;
        private string userName = null;
        private string[] groupMembershipNames = null;
        private string[] groupMembershipSids = null;

        /// <summary>
        /// The Active Directory SID of this subject. 
        /// </summary>
        public string UserSid { get { return userSid; } }

        /// <summary>
        /// The Active Directory DisplayName of the subject.
        /// Null if the lookup failed.
        /// </summary>
        public string UserDisplayName { get { return userDisplayName; } }

        /// <summary>
        /// The Active Directory Name of the subject.
        /// Null if the lookup failed.
        /// </summary>
        public string UserName { get { return userName; } }

        /// <summary>
        /// The Active Directory group names the subject belongs to.
        /// </summary>
        public string[] GroupMembershipNames { get { return groupMembershipNames; } }

        /// <summary>
        /// The Active Directory group sids the subject belongs to.
        /// </summary>
        public string[] GroupMembershipSids { get { return groupMembershipSids; } }

        /// <summary>
        /// Makes server calls, call off the event thread.
        /// </summary>
        /// <param name="session"></param>
        /// <param name="SID"></param>
        private UserDetails(Session session)
        {
            userSid = session.UserSid;
            userDisplayName = GetDisplayName(session);
            userName = GetName(session);
            GetGroupMembership(session);
        }

        private void GetGroupMembership(Session session)
        {
            try
            {
                groupMembershipSids = Auth.get_group_membership(session, userSid);

                if (groupMembershipSids.Length > MAX_GROUP_LOOKUP)
                    return;

                string[] output = new string[groupMembershipSids.Length];


                for (int i = 0; i < groupMembershipSids.Length; i++)
                {
                    string sid = groupMembershipSids[i];
                    Dictionary<String, String> info = Auth.get_subject_information_from_identifier(session, sid);
                    string name = "";

                    if (info.TryGetValue("subject-displayname", out name))
                    {
                        output[i] = name;
                        continue;
                    }
                    if (info.TryGetValue("subject-name", out name))
                    {
                        output[i] = name;
                        continue;
                    }

                    output[i] = sid;

                }
                groupMembershipNames = output;
            }
            catch (Failure)
            {
            }
        }

        private string GetDisplayName(Session session)
        {
            try
            {
                Subject subj = new Subject();
                subj.other_config = Auth.get_subject_information_from_identifier(session, userSid);
                return subj.DisplayName;
            }
            catch (Failure)
            {
                return null;
            }
        }

        private string GetName(Session session)
        {
            try
            {
                Subject subj = new Subject();
                subj.other_config = Auth.get_subject_information_from_identifier(session, userSid);
                return subj.SubjectName;
            }
            catch (Failure)
            {
                return null;
            }
        }
    }
}
