/*
 * Copyright (c) Citrix Systems, Inc.
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of version 2 of the GNU General Public License as published
 * by the Free Software Foundation, with the additional linking exception as
 * follows:
 *
 *   Linking this library statically or dynamically with other modules is
 *   making a combined work based on this library. Thus, the terms and
 *   conditions of the GNU General Public License cover the whole combination.
 *
 *   As a special exception, the copyright holders of this library give you
 *   permission to link this library with independent modules to produce an
 *   executable, regardless of the license terms of these independent modules,
 *   and to copy and distribute the resulting executable under terms of your
 *   choice, provided that you also meet, for each linked independent module,
 *   the terms and conditions of the license of that module. An independent
 *   module is a module which is not derived from or based on this library. If
 *   you modify this library, you may extend this exception to your version of
 *   the library, but you are not obligated to do so. If you do not wish to do
 *   so, delete this exception statement from your version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

using System;
using System.Collections;
using System.Collections.Generic;
using System.Text;

namespace XenAPI
{
    public partial class XenRef<T> where T : XenObject<T>
    {
        private readonly string opaqueRef;

        /// <summary>
        /// 
        /// </summary>
        /// <param name="opaqueRef">May not be null.</param>
        public XenRef(string opaqueRef)
        {
            System.Diagnostics.Trace.Assert(opaqueRef != null, "'opaqueRef' parameter must not be null");
            this.opaqueRef = opaqueRef;
        }

        /// <summary>
        /// Create a XenRef from the opaque_ref of the given XenObject.
        /// </summary>
        /// <param name="obj"></param>
        public XenRef(T obj) : this(obj.opaque_ref)
        {
        }

        /// <param name="opaqueRef">Must be a string.</param>
        public static XenRef<T> Create(object opaqueRef)
        {
            return Create((string)opaqueRef);
        }

        public static XenRef<T> Create(string opaqueRef)
        {
            return new XenRef<T>(opaqueRef);
        }

        public static List<XenRef<T>> Create(string[] opaqueRefs)
        {
            List<XenRef<T>> objs = new List<XenRef<T>>(opaqueRefs.Length);
            foreach (string opaqueRef in opaqueRefs)
            {
                objs.Add(new XenRef<T>(opaqueRef));
            }
            return objs;
        }

        /// <summary>
        /// Creates a List of XenRefs from an array of objects by first calling ToString() on each array element.
        /// </summary>
        /// <param name="opaqueRefs">Must not be null. Must not contain null. May have Length zero. Each element must
        /// return an opaqueRef from its ToString() method.</param>
        /// <returns></returns>
        public static List<XenRef<T>> Create(object[] opaqueRefs)
        {
            List<XenRef<T>> objs = new List<XenRef<T>>(opaqueRefs.Length);
            foreach (object opaqueRef in opaqueRefs)
            {
                objs.Add(new XenRef<T>(opaqueRef.ToString()));
            }
            return objs;
        }

        /// <summary>
        /// Takes a hashmap of string -> hashmap (ie XenAPI ref -> record)
        /// 
        /// Internally convert record to an S (ie a Proxy_VM etc) then uses
        /// that to create a new T (ie a VM) via UpdateFrom
        /// 
        /// returns ie a dict ((ref VM) -> VM)
        /// </summary>
        /// <typeparam name="S"></typeparam>
        /// <param name="o">May not be null.</param>
        /// <returns></returns>
        public static Dictionary<XenRef<T>,T> Create<S>(Object o)
        {
            if (o == null)
                throw new ArgumentNullException("o");

            Hashtable dict = (Hashtable) o;

            Dictionary<XenRef<T>, T> result = new Dictionary<XenRef<T>,T>();

            foreach (Object key in dict.Keys)
            {
                XenRef<T> t_ref = new XenRef<T>((String) key);
                Hashtable t_record = (Hashtable)dict[key];
                result[t_ref] = (T)Marshalling.convertStruct(typeof(T), t_record);
            }

            return result;
        }

        public string opaque_ref
        {
            get 
            {
               return opaqueRef;
            }
        }

        public override bool Equals(object obj)
        {
            XenRef<T> other = obj as XenRef<T>;
            if (other == null)
                return false;
          
            return opaqueRef.Equals(other.opaqueRef);
        }

        public override int GetHashCode()
        {
            return opaqueRef.GetHashCode();
        }

        // convert to an opaqueRef
        public static implicit operator string(XenRef<T> xenRef)
        {
            return xenRef == null ? null : xenRef.opaque_ref;
        }
    }
}
